import json

def export_as_json(path, content):
	"""
	Export data as JSON format.

	Args:
		path: the location of output file.
		content: a dict type variable

	Returns:
		The result of saving.
	"""
	with open(path, 'w+', encoding='utf8') as outfile:
		file_accessed = True
		json.dump(content, outfile, ensure_ascii=False, indent=4)

	if file_accessed:
		return True
	else:
		print("Error: [Export as JSON] Cannot access the file.")
		return False


# -*- coding: utf-8 -*-
# -------------------------------------------
# Nobo, a third-party Manaba API for Ritsumeikan
# 
# manaba.py
# Main Manaba module
# -------------------------------------------
# @Author  : Zhou Fang
# @Updated : 1/28/2019
# @Homepage: https://github.com/fang2hou/Nobo
# -------------------------------------------
import re
import json

from bs4 import BeautifulSoup as bs
from selenium import webdriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.chrome.options import Options

from . import fixja
from . import base

# -------------------------------------------
# Parser function
# -------------------------------------------
def parse_course_info(raw_str):
	raw_str = raw_str.replace(": ",":")
	# Use regex to get the name and code of the course
	info_str_format = r"([0-9]*):(.*)\(([A-Z]|[A-Z][0-9]|[0-9][A-Z])\)"
	code, name, class_order = re.findall(info_str_format, raw_str)[0]
	return code, name, class_order

def parse_course_time(raw_str):
	try:
		# Science
		time_str_format = r"([月|火|水|木|金])([0-9]{1,2})\(([0-9]{1,2})-([0-9]{1,2})\)"
		weekday, period, sci_period_start, sci_period_end = re.findall(time_str_format, raw_str)[0]
	except:
		# Arts
		try:
			time_str_format = r"([月|火|水|木|金])([0-9]{1,2})"
			weekday, period = re.findall(time_str_format, raw_str)[0]
			sci_period_start, sci_period_end = "unknown", "unknown"
		except:
			weekday, period, sci_period_start, sci_period_end = "unknown", "unknown", "unknown", "unknown"

	weekday = fixja.translate_weekday(weekday)
	return weekday, period, sci_period_start, sci_period_end

# NOTICE: This function is unused since campus info has been deleted
def parse_course_room_with_campus(raw_str):
	try:
		time_str_format = r"([衣笠|KIC|BKC|OIC]) ([.*])"
		campus, room  = re.findall(time_str_format, raw_str)[0]
		# Fix if "KIC" written in Kanji.
		campus = campus.replace("衣笠", "KIC")
	except:
		# Other course
		campus, room = "unknown", "unknown"

	return campus, room

# -------------------------------------------
# RitsStudent Class
# -------------------------------------------
class RitsStudent(object):
	def __init__(self, username, password, config_path=None):
		# Initialize user data
		self.username = username
		self.password = password

		# Initialize configuration
		self.config   = base.load_config(path=config_path)
		self.cacheId  = base.convert_to_md5(self.username)
		self.isLogged = False
		
		# Initialize webdriver
		# NOTICE: Enable "headless" in release environment
		chrome_options   = Options()
		# chrome_options.add_argument("--headless")
		self.webdriver   = webdriver.Chrome(chrome_options=chrome_options)
		self.wait_time_out = WebDriverWait(self.webdriver, self.config["manaba"]["timeout"], self.config["manaba"]["login_attempt_interval"])

	def login(self):
		# Try to get the homepage of manaba
		self.webdriver.get(self.config["manaba"]["homepage"])

		# Confirm the current status, reduce the number of unnecessary login operations
		if not self.config["manaba"]["login_domain_root"] in self.webdriver.current_url:
			if self.config["manaba"]["domain_root"] in self.webdriver.current_url:
				# The page is not redirect to the login page, it shows Nobo is in
				print("Nobo: Already login. [User: %s]" % self.username)
				return True
		
		print("Nobo: Try to login... [User: %s]" % self.username)

		# Wait for login button rendering
		try:
			self.wait_time_out.until(lambda sign:self.webdriver.find_element_by_id("web_single_sign-on"))
		except:
			print("Nobo: Login timeout. [User: %s]" % self.username)
			return

		# Enter username
		inputElement = self.webdriver.find_element_by_xpath("//input[@name='USER']")
		inputElement.send_keys(self.username)

		# Enter password
		inputElement = self.webdriver.find_element_by_xpath("//input[@name='PASSWORD']")
		inputElement.send_keys(self.password)

		# Submit the form
		self.webdriver.find_element_by_xpath("//input[@id='Submit']").click()

		# Send a message if username or password is not correct
		if "AuthError" in self.webdriver.current_url:
			print("Nobo: Invalid ID or PASSWORD. [User: %s]")
			return False
	
		return True

	def get_course_list(self):
		if not self.login():
			base.debug_print("Nobo: Login process is failed.")
			return

		base.debug_print("Nobo: Login successful, start to get courses.")
		self.webdriver.get(self.config["manaba"]["homepage"]+"_course?chglistformat=list")
		course_page = self.webdriver.page_source
		course_table_body = bs(course_page, "html.parser").select(".courselist")[0]
		
		# Initialize the output list
		course_list = []

		# Try to get each course information
		# The first -> 0, last 2 -> -2 is not a course (department notice, research etc.)
		base.debug_print("Nobo: Start to parse table of courses.")
		for course_table_line in course_table_body.select(".courselist-c"):

			# Initialize the course
			temp_course = {}

			# Academic year
			# -------------------------------------------
			# If the acdemic year is missed, it shows this line is not a course, maybe a page
			academic_year_tag = course_table_line.find("td").find_next_sibling("td")
			if "" == academic_year_tag.get_text():
				continue
			else:
				academic_year = int(academic_year_tag.get_text())

			# Course name
			# -------------------------------------------
			course_name_tag = course_table_line.find("td")
			# Convert the name into correct encode
			course_name_text = course_name_tag.select(".courselist-title")[0].get_text()
			course_name_text = fixja.convet_to_half_width(course_name_text).strip()

			# If the course has two names and codes, set the flag to process automatically
			if "§" in course_name_text:
				# Split the code, name, and class information
				course_names = course_name_text.split("§")
				course_codes = {}
				course_classes = {}
				course_codes[0], course_names[0], course_classes[0] = parse_course_info(course_names[0])
				course_codes[1], course_names[1], course_classes[1] = parse_course_info(course_names[1])
				temp_course["basic"] = [{
					"code": int(course_codes[0]),
					"name": course_names[0],
					"class": course_classes[0]
				}, {
					"code": int(course_codes[1]),
					"name": course_names[1],
					"class": course_classes[1]
				}]
			else:
				course_code, course_name, course_class = parse_course_info(course_name_text)
				temp_course["basic"] = [{
					"code": int(course_code),
					"name": course_name,
					"class": course_class
				}]

			
			# Course time
			# -------------------------------------------
			# Get the next node that contains course time and classroom information
			course_time_room_tag = academic_year_tag.find_next_sibling("td")
			course_time_text  = course_time_room_tag.find("span").get_text()

			if "春" in course_time_text:
				course_semester = "spring"
			elif "秋" in course_time_text:
				course_semester = "fall"
			else:
				course_semester = "unknown"

			course_weekday, course_period, course_sci_period_start, course_sci_period_end = parse_course_time(course_time_text)

			temp_course["time"] = {
				"year": academic_year,
				"semester": course_semester,
				"weekday": course_weekday,
				"period": course_period,
				"sci_period_start": course_sci_period_start,
				"sci_period_end": course_sci_period_end,
			}

			# Course room
			# -------------------------------------------
			# Delete time tags
			try:				
				course_time_room_tag.span.extract()
				course_time_room_tag.br.extract()
				course_room = course_time_room_tag.get_text().strip()
			except:
				base.debug_print("Nobo: Something wrong with deleting useless tags.")
				course_room = "unknown"
			temp_course["room"] = course_room

			# Course teacher
			# -------------------------------------------
			course_teacher_tag    = course_time_room_tag.find_next_sibling("td")
			course_teacher_text = course_teacher_tag.get_text()

			# Confirm if there are several teachers in list
			if "、" in course_teacher_text:
				course_teachers = course_teacher_text.split("、")
				temp_course["teacher"] = course_teachers
			else:
				course_teacher = [course_teacher_text]
				temp_course["teacher"] = course_teacher

			# Append the information of this course into output list
			course_list.append(temp_course)
		
		base.debug_print("Nobo: Done! [get course list]")
		return course_list


# -*- coding: utf-8 -*-
# Nobo, a third-party Ritsumeikan API
# 
# lib/base.py
#
# The library including some useful functions.
# -------------------------------------------
# @Author  : Fang2hou
# @Updated : 7/31/2018
# @Homepage: https://github.com/fang2hou/Nobo
# -------------------------------------------
import os
import json
import hashlib

DEBUG = True

def load_config(path=None):
	"""
	Load configuration from file.

	The configuration should be a valid JSON file.
	If the path is empty, Nobo will load default configuration.

	Args:
		path: the location that configuration file is. 

	Returns:
		A dict mapping keys to the corresponding table row data fetched. 
	"""

	default_config = {
		"cache_dir": "localdb",
		"manaba": {
			"cookies_cache": True,
			"encryption": "md5",
			"sava_mode": "file"
		}
	}

	if None != path:
		with open(path, 'r') as configuration:
			return json.loads(configuration.read())
	
	return default_config

def LoadCookiesFromFile(cacheDirectory, fileId):
	if not os.path.isdir(cacheDirectory + "/cookies/"):
		os.makedirs(cacheDirectory + "/cookies/")

	try:
		with open(cacheDirectory + "/cookies/" + fileId, 'r+') as f:
			return json.loads(f.read())
	except FileNotFoundError:
		return None

def convert_to_md5(str):
	hashlib.md5().update(str.encode("utf8"))
	return hashlib.md5().hexdigest()

def debug_print(str):
	if DEBUG:
		print(str)


# -*- coding: utf-8 -*-
# Nobo, a third-party Ritsumeikan API
# 
# lib/fixja.py
#
# The library for fixing some Japanese issue.
# -------------------------------------------
# @Author  : Fang2hou
# @Updated : 7/31/2018
# @Homepage: https://github.com/fang2hou/Nobo
# -------------------------------------------
def convet_to_half_width(inputString: str) -> str:
	fixDict = {
		"　": " ",
		"１": "1",
		"２": "2",
		"３": "3",
		"４": "4",
		"５": "5",
		"６": "6",
		"７": "7",
		"８": "8",
		"９": "9",
		"０": "0",
		"（": "(",
		"）": ")",
	}
	finalString = inputString
	for full, half in fixDict.items():
		finalString = finalString.replace(full, half)
	return finalString

def remove_newline(inputString):
	finalString = inputString.replace("\n", "")
	return finalString

def translate_weekday(string):
	convert_dict = {
		"月": "Monday",
		"火": "Tuesday",
		"水": "Wednesday",
		"木": "Thursday",
		"金": "Friday",
		"月曜": "Monday",
		"火曜": "Tuesday",
		"水曜": "Wednesday",
		"木曜": "Thursday",
		"金曜": "Friday",
		"月曜日": "Monday",
		"火曜日": "Tuesday",
		"水曜日": "Wednesday",
		"木曜日": "Thursday",
		"金曜日": "Friday",
	}

	for jp, en in convert_dict.items():
		string = string.replace(jp, en)

	return string

def remove_last_space(string):
	remove_list = [
		"\xa0", # Full-width Space used in Japanese
		"\n",
		"\t",
		" "
	]

	if string in remove_list:
		string = ""
	else:
		while string[-1:] in remove_list:
			string = string[:-1]

	return string


#!/usr/bin/env python

from distutils.core import setup

setup(name='nobo',
      version='0.1.2',
      description='A third-party Rits API',
      long_description='A third-party spider that you could get your data from each service provides by Ritsumeikan University',
      author='Zhou Fang',
      license='GPLv3',
      author_email='is0385rx@ed.ritsumei.ac.jp',
      url='https://github.com/fang2hou/Nobo',
      packages=['Nobo'],
      keywords = ['ritsumei', 'manaba'],
      python_requires='>=3.4',
      install_requires=[
        "beautifulsoup4",
        "selenium",
      ],
     )


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ----------------------------------------
# Syllabus module of Nobo.
# Fang2hou @ 2/7/2018
# github: https://github.com/fang2hou/Nobo
# ----------------------------------------
import requests
import re
import json
from bs4 import BeautifulSoup as bs
from .lib import fixja

class syllabusUser(object):
	"""syllabus Class"""
	# Setting for syllabus
	syllabusHomePagePath = "https://campusweb.ritsumei.ac.jp/syllabus/sso/KJgSearchTop.do"

	def __init__(self, username, password):
		self.rainbowID = username
		self.rainbowPassword = password
		self.webSession = requests.Session()
		self.syllabusList = {}

	def login(self):
		# Load first page using the webSession initialized before
		firstPage = self.webSession.get(self.syllabusHomePagePath)
		# Initialize the post data for second page
		secondPagePostData = {
			"USER": self.rainbowID,
			"PASSWORD": self.rainbowPassword
		}
		# Update the post data using the given information
		for inputElement in bs(firstPage.text, "html.parser").find_all('input'):
			tempAttrDict = inputElement.attrs
			if "name" in tempAttrDict and "value" in tempAttrDict:
				if tempAttrDict["name"] == "target":
					secondPagePostData["target"] = tempAttrDict["value"]
				elif tempAttrDict["name"] == "smauthreason":
					secondPagePostData["smauthreason"] = tempAttrDict["value"]
				elif tempAttrDict["name"] == "smquerydata":
					secondPagePostData["smquerydata"] = tempAttrDict["value"]
				elif tempAttrDict["name"] == "postpreservationdata":
					secondPagePostData["postpreservationdata"] = tempAttrDict["value"]
		
		# Load second page
		secondPage = self.webSession.post("https://sso.ritsumei.ac.jp/cgi-bin/pwexpcheck.cgi", secondPagePostData)
		# Get the redirect path
		thirdPagePath = "https://sso.ritsumei.ac.jp" + bs(secondPage.text, "html.parser").find('form').attrs["action"]
		# Create the post data for second page
		thirdPagePostData = {
			"USER": self.rainbowID,
			"PASSWORD": self.rainbowPassword,
			"smquerydata": secondPagePostData["smquerydata"],
		}

		# Load third page
		self.webSession.post(thirdPagePath, thirdPagePostData)

	def getSyllabusById(self, courseYear, courseID):
		self.syllabusList = {}

		syllabusAllInfoPagePath = "https://syllabus.ritsumei.ac.jp/syllabus/sso/SyShowAll.do"

		# Get all information about this course with (sylNendo, jyugyocd, kinouID)
		postData = {
			"mkjgkbcd": "",
			"daibuncd": "",
			"tyubuncd": "",
			"shobuncd": "",
			"sylNendo": courseYear,
			"jyugyocd": courseID,
			"kinouID": 'SKJ070',
		}

		# Save page source
		syllabusPage = self.webSession.post(syllabusAllInfoPagePath, postData)
		syllabusPageHeadTag = bs(syllabusPage.text, "html.parser")

		# FOR DEBUG
		# ---------------------------------
		# # get file and save -------------
		# syllabusPage = self.webSession.post(syllabusAllInfoPagePath, postData)
		# with open("temp.html", "w+") as f:
		# 	f.write(syllabusPage.text)
		# return 

		# # fake data ---------------------
		# with open("temp.html", "r") as inputFile:
		# 	syllabusPageHeadTag = bs(inputFile.read(), "html.parser")

		# Start of processing
		# Basic
		basicInfoTag = syllabusPageHeadTag.select(".jugyo_table")[0].find("tr").find_next_sibling("tr").find("td")
		courseNameString = fixja.remove_last_space(basicInfoTag.get_text().strip("\t\n "))
		courseNameString = fixja.convet_to_half_width(courseNameString)
		basicInfo = []

		if "  " in courseNameString:
			courseNames = courseNameString.split("  ")
			for courseName in courseNames:
				courseClass = re.findall(r"\([A-Za-z][0-9]\)", courseName)[0].strip("()")
				courseName = courseName.split(" (")[0]
				basicInfo.append({
					"name": courseName,
					"class": courseClass
				})
		else:
			courseName = courseNameString
			courseClass = re.findall(r"\([A-Za-z][0-9]\)", courseName)[0].strip("()")
			courseName = courseName.split(" (")[0]
			basicInfo.append({
				"name": courseName,
				"class": courseClass
			})

		# Semester
		semesterTag = basicInfoTag.find_next_sibling("td")
		semesterString = semesterTag.get_text()
		if "前期" in semesterString:
			courseSemester = "spring"
		elif "後期" in semesterString:
			courseSemester = "fall"
		else:
			courseSemester = "unknown"

		# Course Time
		courseTimeTag = semesterTag.find_next_sibling("td")
		courseTimeString = courseTimeTag.get_text().replace("\t", "").replace("\n", "")
		courseWeekday, courseArtPeriod, courseSciPeriod = re.findall(r"([月|火|水|木|金])([0-9]-[0-9]|[0-9])(\([0-9]-[0-9]\))", courseTimeString)[0]
		courseWeekday = fixja.convert_week_to_en(courseWeekday)
		courseSciPeriod = courseSciPeriod.strip("()")
		courseTime = {
			"year": courseYear,
			"semester": courseSemester,
			"weekday": courseWeekday,
			"period_art": courseArtPeriod,
			"period_sci": courseSciPeriod,
		}
		
		# Credit
		creditTag = courseTimeTag.find_next_sibling("td")
		credit = int(creditTag.get_text())

		# Teacher
		courseTeacherTag = creditTag.find_next_sibling("td")
		courseTeacherString = courseTeacherTag.get_text()
		courseTeacherString = fixja.remove_last_space(courseTeacherString.replace("\t", "").replace("\n", "").strip(" "))

		# Confirm if there are several teachers in list
		if "、" in courseTeacherString:
			hasSeveralTeachers = True
		else:
			hasSeveralTeachers = False

		if hasSeveralTeachers:
			courseTeacher = courseTeacherString.split("、")
		else:
			courseTeacher = [courseTeacherString]

		# Course Outline and Method
		outlineTag = syllabusPageHeadTag.find("dl")
		outline = fixja.remove_last_space(outlineTag.find("dd").get_text())
		
		# Student Attainment Objectives
		objectivesTag = outlineTag.find_next_sibling("dl")
		objectives = ""
		for subString in objectivesTag.find("dd").find_next_sibling("dd").contents:
			if str(subString) != "<br/>":
				objectives += subString.replace("\n", "").replace("\t", "")
			else:
				objectives += "\n"
		
		# Recommended Preparatory Course
		preCourseTag = objectivesTag.find_next_sibling("dl")
		precourse = fixja.remove_last_space(preCourseTag.find("dd").get_text())

		# Course Schedule
		scheduleTableTag = preCourseTag.find_next_sibling("dl")
		tempScheduleTableTag = scheduleTableTag.find("table").find("tr").find_next_sibling("tr")

		scheduleList = []

		while tempScheduleTableTag:
			# Lecture
			tempSchedule = {"lecture": int(tempScheduleTableTag.find("td").get_text()), "theme": fixja.remove_last_space(
				fixja.convet_to_half_width(tempScheduleTableTag.find("td").find_next_sibling("td").get_text()))}
			# Theme
			tempScheduleTableTag = tempScheduleTableTag.find_next_sibling("tr")
			# Keyword, References and Supplementary Information
			tempSchedule["references"] = fixja.remove_last_space(fixja.convet_to_half_width(tempScheduleTableTag.find("td").get_text()))
			tempScheduleTableTag = tempScheduleTableTag.find_next_sibling("tr")

			scheduleList.append(tempSchedule)
		
		# Recommendations for Private Study
		recommendationTag = scheduleTableTag.find_next_sibling("dl")
		recommendation = fixja.remove_last_space(recommendationTag.find("dd").get_text())
		
		# Grade Evaluation Method
		gradeEvaluationTag = recommendationTag.find_next_sibling("dl")
		gradeEveluationTableTag = gradeEvaluationTag.find("table")

		gradeEvaluation = {
			"note": "",
			"data": [],
		}

		gradeType = [
				"End of Semester Examination (Written)",
				"Report Examination",
				"Other"
		]

		gradeTypeIndex = 0

		for row in gradeEveluationTableTag.find_all("tr")[1:]:
			percentage = int(row.select(".percentage")[0].get_text().replace("%", ""))
			gradeNote = fixja.remove_last_space(row.select(".top")[0].get_text())
			gradeEvaluation["data"].append({
				"type": gradeType[gradeTypeIndex],
				"percentage": percentage,
				"note": gradeNote
			})
			gradeTypeIndex += 1

		gradeEvaluationNoteTag = gradeEveluationTableTag.find_next_sibling("dl")
		gradeEvaluationNote = gradeEvaluationNoteTag.find("dd").get_text()
		gradeEvaluationNote = fixja.remove_last_space(gradeEvaluationNote)
		gradeEvaluation["note"] = gradeEvaluationNote

		# Advice to Students on Study and Research Methods
		adviceTag = gradeEvaluationTag.find_next_sibling("dl")
		advice = fixja.remove_last_space(adviceTag.find("dd").get_text())

		# Textbooks
		textBookTag = adviceTag.find_next_sibling("dl")
		textBooks = {
			"note": "",
			"book": []
		}

		textBooksTable = textBookTag.find("table")
		if textBooksTable:
			for row in textBooksTable.find_all("tr")[1:]:
				tempTextBook = {}
				tempTag = row
				tempTag = tempTag.find("td")
				tempTextBook["title"] = fixja.remove_last_space(tempTag.get_text()).replace("\n", "")
				tempTag = tempTag.find_next_sibling("td")
				tempTextBook["author"] = fixja.remove_last_space(tempTag.get_text())
				tempTag = tempTag.find_next_sibling("td")
				tempTextBook["publisher"] = fixja.remove_last_space(tempTag.get_text())
				tempTag = tempTag.find_next_sibling("td")
				tempTextBook["ISBN"] = fixja.remove_last_space(tempTag.get_text()).replace("ISBN","")
				tempTag = tempTag.find_next_sibling("td")
				tempTextBook["comment"] = fixja.remove_last_space(tempTag.get_text())
				textBooks["book"].append(tempTextBook)
		textBooks["note"] = fixja.remove_last_space(textBookTag.find_all("dd", class_="nest")[0].get_text())

		# Reference Books
		refBookTag = textBookTag.find_next_sibling("dl")
		refBooks = {
			"note": "",
			"book": []
		}

		refBooksTable = refBookTag.find("table")
		if refBooksTable:
			for row in refBooksTable.find_all("tr")[1:]:
				tempRefBook = {}
				tempTag = row
				tempTag = tempTag.find("td")
				tempRefBook["title"] = fixja.remove_last_space(tempTag.get_text()).replace("\n", "")
				tempTag = tempTag.find_next_sibling("td")
				tempRefBook["author"] = fixja.remove_last_space(tempTag.get_text())
				tempTag = tempTag.find_next_sibling("td")
				tempRefBook["publisher"] = fixja.remove_last_space(tempTag.get_text())
				tempTag = tempTag.find_next_sibling("td")
				tempRefBook["ISBN"] = fixja.remove_last_space(tempTag.get_text()).replace("ISBN","")
				tempTag = tempTag.find_next_sibling("td")
				tempRefBook["comment"] = fixja.remove_last_space(tempTag.get_text())
				refBooks["book"].append(tempRefBook)
		refBooks["note"] = fixja.remove_last_space(refBookTag.find_all("dd", class_="nest")[0].get_text())
		
		# Web Pages for Reference
		refPageTag = refBookTag.find_next_sibling("dl")
		refPageStrings = refPageTag.find("dd").contents
		refPage = ""

		for refPageString in refPageStrings:
			afterString = fixja.remove_last_space(str(refPageString))
			if afterString != "":
				if "<a" in afterString:
					refPage += re.findall(r">(.+)<", afterString)[0]
					refPage += " "
				elif afterString == "<br/>":
					refPage = refPage[:-1] + "\n"
				else:
					refPage += afterString
					refPage += " "

		refPage = fixja.remove_last_space(refPage)

		# How to Communicate with the Instructor In and Out of Class(Including Instructor Contact Information)
		contactTag = refPageTag.find_next_sibling("dl")
		contactString = contactTag.find("dd")
		contactMethods = []

		for contactMethod in re.findall(r"<b>(.*)／", str(contactString)):
			contactMethods.append(fixja.remove_last_space(contactMethod))

		# Other Comments
		otherCommentsTag = contactTag.find_next_sibling("dl")
		otherComments = fixja.remove_last_space(otherCommentsTag.find("dd").get_text())

		# Save as dictionary
		self.syllabusList = {
			"basic": basicInfo,
			"time": courseTime,
			"teacher": courseTeacher,
			"credit": credit,
			"outline": outline,
			"objectives": objectives,
			"precourse": precourse,
			"schedule": scheduleList,
			"recommendation": recommendation,
			"grade_evluation": gradeEvaluation,
			"advice": advice,
			"text_books": textBooks,
			"ref_books": refBooks,
			"ref_pages": refPage,
			"contact_methods": contactMethods,
			"other_comments": otherComments,
		}

	def outputAsJSON(self, outputPath):
		if len(self.syllabusList) > 0:
			# Output data if the user has got information of all courses
			with open(outputPath, 'w+', encoding='utf8') as outfile:
				# Fix Kanji issue, set indent as 4
				json.dump(self.syllabusList, outfile, ensure_ascii=False, indent=4)
		else:
			# Notify when output without information of courses
			print("Use the getSyllabusById() method to get data first.")